/* Note this is a generated file.
Before editing this file directly consider editing the script
generateFilterCode.pl
which generates the idl file and much of the required javascript
code for an XPCOM component
*/

/***********************************************************
constants
***********************************************************/

// reference to the interface defined in cmguiIDiscreteGaussianFilter.idl
const cmguiIDiscreteGaussianFilter = Components.interfaces.cmguiIDiscreteGaussianFilter;

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;

// references to other required interfaces
// const nsIDOMWindow = Components.interfaces.nsIDOMWindow;

// 6599D662-C07A-11DB-9B74-1BCC55D89593 uniquely identifying our component
const CLASS_ID = Components.ID("{6599D662-C07A-11DB-9B74-1BCC55D89593}");

// description
const CLASS_NAME = "Cmgui discrete gaussian filter XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@bioeng.auckland.ac.nz/discretegaussianfilter;1";

/***********************************************************
class definition
***********************************************************/


//class constructor
function DiscreteGaussianFilter() {
};



// class definition
DiscreteGaussianFilter.prototype = {

  // members
	parent: null,
	commandData: null,
	variance: 1,
	maxKernelWidth: 4,
	inputFieldName: null,
  inputField: null,
	outputFieldName: null,
  outputField: null,
  callback: null,
	//	resampledInputFieldName: null,
	//	resampledInputField: null,
	previewOutputFieldName: null,
	previewOutputField: null,
	gElementName: null,
	sceneViewer: null,
	previewResolution: 128,  // should be a power of 2
	//	maxPreviewResolution: 256,
// define the function methods we want to expose in our interface

  init: function(parentWindow, commandData, inputFieldName, outputFieldName, callback) {

		this.parent = parentWindow;
		this.commandData = commandData;         // command data object for accessing cmgui commands    
		this.inputFieldName = inputFieldName;	
		this.outputFieldName = outputFieldName;
		this.callback = callback;
		
		// find the input field, note it is assumed to exist
	  this.inputField = commandData.findFieldByName(inputFieldName);
		
		// set some defaults which will be overwritten if the field exists
		this.variance = 1;
		this.maxKernelWidth = 4;
		
		// check if output field exists
		if( commandData.isFieldDefined(outputFieldName) ){

			// field exists so fetch it
			this.outputField = commandData.findFieldByName(outputFieldName);
			
			// create some objects which will be populated by the current field
			// properties if the field is the right type
			
			var returnInputField = new Object();
			var variance = new Object();
			var maxKernelWidth = new Object();					
			
			try {
				
				this.outputField.getTypeDiscreteGaussianImageFilter(returnInputField,variance,maxKernelWidth);
				
				// when fetched the parameters are assigned to the value property 
				this.variance = variance.value;
				this.maxKernelWidth = maxKernelWidth.value;
				
			} catch(ex) {

				this.parent.alert("Could not fetch parameters, probably because " + 
													"the output field exists but is of the wrong type " + ex);				
				this.outputField.setTypeDiscreteGaussianImageFilter(this.inputField,this.variance,this.maxKernelWidth);

			}
			
		} else {

			// field does not exist so create it.

			this.outputField = commandData.createField(outputFieldName);
			this.outputField.setTypeDiscreteGaussianImageFilter(this.inputField,this.variance,this.maxKernelWidth);
			
		}

  },


	setup_preview_field: function() {


		// first resample the input field
		// will eventually use createField and setType to do this
		//		this.resampledInputFieldName = "resampled_" + this.inputFieldName;
		//		this.commandData.executeCommand("gfx def field " + this.resampledInputFieldName +
		//															 " image_resample field " + this.inputFieldName + " sizes 128 128;");
		//		this.resampledInputField = this.commandData.findFieldByName(this.resampledInputFieldName);
		
 		// Set up previewOutputField
		this.previewOutputFieldName = "preview_" + this.outputFieldName;
		//		this.previewOutputField = this.commandData.createField(this.previewOutputFieldName);
			
		// set the type with the current options
		//		this.previewOutputField.setTypeDiscreteGaussianImageFilter(this.resampledInputField,this.variance,this.maxKernelWidth);

		this.commandData.executeCommand("gfx def field " + this.previewOutputFieldName +
  		" image_resample field " + this.outputFieldName + 
      " sizes " + this.previewResolution + " " + this.previewResolution + ";");

	},


	resample_preview: function() {

		this.commandData.executeCommand("gfx def field " + this.previewOutputFieldName +
			" image_resample field " + this.outputFieldName + 
      " sizes " + this.previewResolution + " " + this.previewResolution + ";");

	},


	// call this function from the dialog to take care of updating the filter and running the callback
  apply_filter: function() {

		this.outputField.setTypeDiscreteGaussianImageFilter(this.inputField,this.variance,this.maxKernelWidth);			
		this.callback.run();

  },

	// should this return the name of the material?  change this to update_preview_field
  preview_filter: function() {

		// update the preview output field options
		this.outputField.setTypeDiscreteGaussianImageFilter(this.inputField,this.variance,this.maxKernelWidth);			

  },


	create_dialog: function() {

		var parent = this.parent;
		var params={discreteGaussianFilter:this};

		this.setup_preview_field();

		try {
						
      dialogReference = parent.openDialog("chrome://zinc/content/filters/discreteGaussianFilterDialog.xul",
																					"discrete-gaussian-filter-dialog",
																					"chrome",params);
			
			// store reference to dialog and associated field name
			parent.discreteGaussianFilterDialogReference = dialogReference;
			parent.discreteGaussianFilterDialogReferenceOutputField = this.outputFieldName;

		}catch(ex) {
			parent.alert("Error creating dialog to edit discrete gaussian filter: " + ex);
		}

	},
	

  open_dialog: function(gElementName, sceneViewer) {

		this.gElementName = gElementName;
		this.sceneViewer = sceneViewer;

		// window parent
		var parent = this.parent;
		var dialogReference = parent.discreteGaussianFilterDialogReference;

		// determine if a dialog already exists
		if (dialogReference == null || dialogReference.closed) {
			
			// no dialog so create one
			this.create_dialog();
			
		} else { 
			
			// dialog exists, check if it is relevant to this field.				
			var oldOutputFieldName = parent.discreteGaussianFilterDialogReferenceOutputField;
			
			if (oldOutputFieldName == this.outputFieldName) {
				
				// correct dialog so bring it to the front
				dialogReference.focus();
				
			} else {
				
				// not correct so close dialog and open a new one
				dialogReference.close();
				this.create_dialog();

			}
		}
		
  },
	
    

  // the QueryInterface function throws an exception
  // if the cmguiIGeneralThresholdFilter interface is not implemented
  // otherwise it returns a reference to the GeneralThresholdFilter object
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(cmguiIDiscreteGaussianFilter) &&    
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};


/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. Eg:

myDiscreteGaussianFilter = Components.classes["@bioeng.auckland.ac.nz/helloworld;1"].
                          createInstance(Components.interfaces.cmguiIDiscreteGaussianFilter);

***********************************************************/
var DiscreteGaussianFilterFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new DiscreteGaussianFilter()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var DiscreteGaussianFilterModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, 
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return DiscreteGaussianFilterFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return DiscreteGaussianFilterModule; }
